package cn.fantix.phpftp;

import rheise.jftpd.Logger;
import rheise.jftpd.ParseException;

import java.io.*;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Properties;

/**
 * This is the main class for jftpd. It creates the server socket and loops
 * forever accepting user connections and creating ServerPI threads to handle
 * them.
 * <p/>
 * To run under unix, type:
 * <p/>
 * <pre>java cn.fantix.phpftp.PhpFtpServer</pre>
 */
public class PhpFtpServer
        //implements org.jos.services.common.PhpFtpServer
{
    /**
     * The version number of jftpd.
     */
    public static final String VERSION = "Version 0.3";

    /**
     * The default data port.
     */
    public static final int SERVER_DATA_PORT = 20;

    /**
     * The port this server is listening on.
     */
    private int port;

    private String phpUrl = "http://www.fantix.cn/phpftp/server.php?ftp_home=../";

    public static void main(String[] args)
            throws Exception {
        if (args.length > 1) {
            System.err.println("Usage: PhpFtpServer [ port ]");
            System.exit(1);
        }


        InputStream propertiesIn = new FileInputStream("phpftp.properties");
//        if (propertiesIn == null)
//            throw new FileNotFoundException("${archive}/cn/fantix/phpftp/phpftp.properties");
        Properties properties = new Properties();
        properties.load(propertiesIn);

        int port;
        PhpFtpServer server;
        if (args.length == 1) {
            port = Integer.parseInt(args[0]);
            server = new PhpFtpServer(properties, port);
        } else {
            server = new PhpFtpServer(properties);
        }

        // Start the FTP server as a standalone server.
        //
        server.start();
    }

    /**
     * Create an FTP server to run on port 21.
     *
     * @param properties properties
     * @throws rheise.jftpd.ParseException ParseException
     */
    public PhpFtpServer(Properties properties)
            throws ParseException {
        this(properties, -1);
    }

    /**
     * Create an FTP server to run on the specified port.
     *
     * @param properties properties
     * @param port       port
     * @throws rheise.jftpd.ParseException ParseException
     */
    public PhpFtpServer(Properties properties, int port)
            throws ParseException {
        this.port = port;
        configure(properties);
    }

    /**
     * Starts the FTP server. This method listens for connections on the
     * FTP server port (usually port 21), and spawns a new thread to
     * handle each connection.
     *
     * @throws Exception exception
     */
    private void start()
            throws Exception {
        ServerSocket serverSocket = new ServerSocket(port);
        Logger.log(Logger.LOG_INFO, "PhpFtpServer starts listening at port " + port);
        //noinspection InfiniteLoopStatement
        while (true) {
            Socket clientSocket = serverSocket.accept();
            ServerPI pi = new ServerPI(clientSocket, phpUrl);
            new Thread(pi).start();
        }
    }

    /**
     * Handle a client connection. This may be called directly by a
     * superserver instead of start().
     *
     * @param socket the client socket.
     * @throws Exception exception
     */
    public void service(Socket socket)
            throws Exception {
        ServerPI pi = new ServerPI(socket, phpUrl);
        pi.run();
    }

    /**
     * Configures the ftp server from the configuration properties.
     *
     * @param properties properties
     * @throws rheise.jftpd.ParseException ParseException
     */
    private void configure(Properties properties)
            throws ParseException {
        if (port == -1) {
            String portstr = properties.getProperty("server.port");
            try {
                port = Integer.parseInt(portstr);
            } catch (NumberFormatException e) {
                throw new ParseException(ParseException.PROPERTY_INVALID, "server.port", portstr);
            }
        }

        String priorityStr = properties.getProperty("log.priority");
        if (priorityStr == null)
            throw new ParseException(ParseException.PROPERTY_UNDEFINED, "log.priority", null);
        String priorityNames[] = Logger.priorityNames;
        int priority = Logger.LOG_OFF;
        for (int i = 0; i < priorityNames.length; i++) {
            if (priorityNames[i].equals(priorityStr)) {
                priority = i;
                break;
            }
        }

        Logger.setPriority(priority);

        String output = properties.getProperty("log.output");
        OutputStream outputStream;
        if (output == null || output.equals("stderr")) {
            outputStream = System.err;
        } else {
            try {
                outputStream = new FileOutputStream(output, true);
            }
            catch (IOException e) {
                throw new ParseException(ParseException.PROPERTY_INVALID, "log.output", output);
            }
        }
        Logger.setWriter(new PrintWriter(new OutputStreamWriter(outputStream), true));
    }
}
